#!/usr/bin/perl

##############################################################
#                                                            #
# feiertage.pl V1.0 17.01.2012                               #
#                                                            #
# berechnet auf Basis der Gaußchen Osterformel Ostersonntag, #
# und darauf basierend andere christl. Feiertage             #
#                                                            #
# basiert auf der Gaußschen Osterformel:                     #
# "http://de.wikipedia.org/wiki/Gau%C3%9Fsche_Osterformel"   #
#                                                            #
# dirk@schimansky.com                                        #
#                                                            #
##############################################################

use Time::Local;

###
# für welches Jahr?
my $YEAR=$ARGV[0];
if (length "$YEAR" != "4"){
	print "\n  Kein Jahr (mit 4 Stellen) angegeben --> Abbruch!\n";
	print "\n  Aufruf mit: perl feiertage.pl XXXX\n\n";
	exit;
}

###
# fuer Zufalls-ID: kleines Alphabet mit Zahlen
my @ID_ELEMENTS=("a".."f","0".."9");

###
# Ostern ermitteln
my $A=$YEAR % 19;
my $B=int($YEAR / 100);
my $C=int((8*$B+13)/25)-2;
my $D = $B - int($YEAR/400) - 2;
my $E = (19 * ($YEAR % 19) + ((15 - $C + $D) % 30)) % 30;
if ($E == 28){
	if ($A > 10){
		$E = 27;
	}
} elsif ($E == 29){
	$E = 28;
}
my $F = ($D + 6 * $E + 2 * ($YEAR % 4) + 4 * ($YEAR % 7) + 6) % 7;
my $ostersonntag=timelocal(0,0,0,1,2,$YEAR)+($E+$F+21)*86400;

###
# von Ostern abhängige Feiertage
$feiertag{$ostersonntag}="Ostersonntag";
$feiertag{$ostersonntag-2*86400}="Karfreitag";
$feiertag{$ostersonntag+1*86400}="Ostermontag";
$feiertag{$ostersonntag+39*86400}="Christi Himmelfahrt";
$feiertag{$ostersonntag+49*86400}="Pfingstsonntag";
$feiertag{$ostersonntag+50*86400}="Pfingstmontag";
$feiertag{$ostersonntag+60*86400}="Fronleichnam";

###
# Karneval
$feiertag{$ostersonntag-52*86400}="Weiberdonnerstag";
$feiertag{$ostersonntag-48*86400}="Rosenmontag";

###
# Feste Feiertage (Monat-1 angeben!)
$feiertag{timelocal(0,0,0,1,0,$YEAR)}="Neujahr";
$feiertag{timelocal(0,0,0,1,4,$YEAR)}="Maifeiertag";
$feiertag{timelocal(0,0,0,3,9,$YEAR)}="Deutsche Einheit";
$feiertag{timelocal(0,0,0,1,10,$YEAR)}="Allerheiligen";
$feiertag{timelocal(0,0,0,24,11,$YEAR)}="Hl. Abend";
$feiertag{timelocal(0,0,0,25,11,$YEAR)}="1. Weihnachtstag";
$feiertag{timelocal(0,0,0,26,11,$YEAR)}="2. Weihnachtstag";
$feiertag{timelocal(0,0,0,31,11,$YEAR)}="Silvester";

###
# Adventzeit bestimmen (4. Advent ist der letzte Sonntag vor dem 25.12.)
my $wochentag_25_12=&timetodate(timelocal(0,0,0,25,11,$YEAR),2);
if ($wochentag_25_12 == 0){	# 25. ist ein Sonntag
	$advent4=timelocal(0,0,0,25-7,11,$YEAR);
	$feiertag{$advent4}="4. Advent";
} elsif ($wochentag_25_12 == 1){	# 25. ist ein Montag --> 4. Adv. und Weihnachten fallen zusammen
	$advent4=timelocal(0,0,0,25-$wochentag_25_12,11,$YEAR);
	$feiertag{$advent4}="4. Advent & Hl. Abend";
} else {
	$advent4=timelocal(0,0,0,25-$wochentag_25_12,11,$YEAR);
	$feiertag{$advent4}="4. Advent";
}

$feiertag{$advent4-7*86400}="3. Advent";
$feiertag{$advent4-14*86400}="2. Advent";
$feiertag{$advent4-21*86400}="1. Advent";

###
# Sommerzeit
$wochentag_CEST=&timetodate(timelocal(0,0,0,31,2,$YEAR),2);
$CEST=timelocal(0,0,0,31-$wochentag_CEST,2,$YEAR)+7200;	# um 2:00 Vorstellung auf 3:00
$feiertag{$CEST}="Sommerzeit";
$feiertag_description{$CEST}="Um 2:00 Uhr wird auf 3:00 Uhr vor gestellt";

###
# Winterzeit
$wochentag_CET=&timetodate(timelocal(0,0,0,31,9,$YEAR),2);
$CET=timelocal(0,0,0,31-$wochentag_CET,9,$YEAR)+10800;	# um 3:00 Rueckstellung auf 2:00
$feiertag{$CET}="Winterzeit";
$feiertag_description{$CET}="DESCRIPTION:Um 3:00 Uhr wird auf 2:00 Uhr zurueck gestellt";

###
# eigene Tage
$feiertag{timelocal(0,0,0,6,0,$YEAR)}="Hl. 3 Könige";
$feiertag{timelocal(0,0,0,14,1,$YEAR)}="Valentinstag";
$feiertag{timelocal(0,0,0,31,9,$YEAR)}="Halloween";

###
# Ausgabe
&write_ics;	# fuer davical
&output;	# einfach auf Konsole ausgeben

###
# ich habe fertig!
exit;

###########################################################################
###########################################################################
###########################################################################

sub timetodate {
	my $date="";
	my $type=$_[1];
	($sec,$min,$hour,$mday,$month,$YEAR,$wday,$yday,$isdst) = localtime($_[0]);
	$month = $month+1;
	$hour = "0$hour" if ($hour < 10);
	$min = "0$min" if ($min < 10);
	$sec = "0$sec" if ($sec < 10);
	$YEAR += 1900;
	$month = "0$month" if ($month < 10);
	$mday = "0$mday" if ($mday < 10);
	$YEAR = "0$YEAR" if ($YEAR < 10);

	if ($type==1){	# der Tag für ics-Datei
		return("$YEAR$month$mday"."T"."$hour$min$sec"."Z");
	} elsif ($type==2){	# fuer Adventsberechnung (auf welchen Tag fällt der 1. Weihnachtstag?)
		return("$wday");
	} elsif ($type==3){	# fuer normale Ausgabe in output
		return("$YEAR-$month-$mday $hour:$min:$sec");
	}
}

sub write_ics {

	open F,">feiertage_$YEAR.ics";
	print F "BEGIN:VCALENDAR\r\n";
	foreach $day(sort keys %feiertag) {
		print F "BEGIN:VEVENT\r\n";
		print F "CREATED:",&timetodate(time,1),"\r\n";
		print F "LAST-MODIFIED:",&timetodate(time,1),"\r\n";
		print F "DTSTAMP:",&timetodate($day,1),"\r\n";
		print F "UID:$YEAR$month$mday-",&ID(4),"-",&ID(4),"-",&ID(4),"-",&ID(12),"\r\n";
		print F "SUMMARY:$feiertag{$day}\r\n";
		print F "DTSTART;VALUE=DATE:$YEAR$month$mday\r\n";
		&timetodate($day+86400,1);
		print F "DTEND;VALUE=DATE:$YEAR$month$mday\r\n";
		if (defined("$feiertag_description{$day}")){
			print F "DESCRIPTION:$feiertag_description{$day}\r\n";
		}
		print F "END:VEVENT\r\n";
	}
	print F "END:VCALENDAR\r\n";
	close F;
}

sub output {
	foreach $day(sort keys %feiertag) {
		print " --> \$day=\"$day\":",&timetodate($day,3)," --> $feiertag{$day}\n";
	}
}

sub ID {
	my $ID,$i;
	for ($i=0;$i<$_[0];$i++){
		$ID.="$ID_ELEMENTS[int(rand($#ID_ELEMENTS) + 1)]";
	}
	return ${ID};
}

