#!/bin/sh

###################################################################
#
# Trägt mit feiertage.pl errechnete Feiertage des folgenden Jahres
# in den davical-Kalender ein.
#
# Aufruf als Cron-Job:
# 25 2 1 1 *      root  /home/davical/feiertage_eintragen.sh
#
# 10.09.2011 - dirk@schimansky.com
#
###################################################################

###
# Einstellungen
ADMIN="admin"
PW="geheim"
SERVER=`hostname -f`

###
# Verzeichnis der Kalenderdaten
DAVICALDIR="/home/davical"

###
# Log
LOG="${DAVICALDIR}/feiertage_eintragen.log"

###
# Jahr ermitteln (läuft in Neujahrsnacht, also aktuelles Jahr+1)
echo -n "`date +%T`: ermittle Jahr, das eingetragen werden soll: " >> ${LOG}
DAVICALYEAR=$((`date +%Y`+1))
echo "${DAVICALYEAR}" >> ${LOG}

###
# prüfen, ob Daten für das Jahr schon eingetragen wurden
if [ -f "${DAVICALDIR}/feiertage_${DAVICALYEAR}.ics" ];then
	echo ""
	echo "  \"${DAVICALDIR}/feiertage_${DAVICALYEAR}.ics\" existiert schon, gehe also"
	echo "  davon aus, dass Daten für ${DAVICALYEAR} schon in davical eingetragen sind"
	echo "  --> Abbruch!"
	echo ""
	exit
fi

###
# prüfen, ob schon gelaufen
echo "`date +%T`: prüfe, ob ${DAVICALDIR}/feiertage_${DAVICALYEAR}.ics vorhanden" >> ${LOG}
[ -f "${DAVICALDIR}/feiertage_${DAVICALYEAR}.ics" ] && echo "${DAVICALDIR}/feiertage_${DAVICALYEAR}.ics schon vorhanden --> Abbruch!" && exit

###
# Feiertage ermitteln
echo "`date +%T`: Feiertage berechnen: perl ${DAVICALDIR}/feiertage.pl ${DAVICALYEAR}" >> ${LOG}

perl ${DAVICALDIR}/davical/feiertage.pl ${DAVICALYEAR}

###
# raus wenn kein Kalender definiert oder Kalender nicht vorhanden ist
echo "`date +%T`: Feiertage in davical eintragen" >> ${LOG}
curl --insecure --basic --request PUT \
	--header "Content-Type: text/calendar; charset=utf-8" \
	--user ${ADMIN}:${PW} \
	--data-binary @"${DAVICALDIR}/feiertage_${DAVICALYEAR}.ics" \
	https://${SERVER}:8443/caldav.php/gemeinsam/home/?mode=append > /dev/null 2>&1

echo "`date +%T`: fertig!" >> ${LOG}
echo "" >> ${LOG}

###
# Mail schreiben
(
    echo "To: root"
    echo "From: root"
    echo "Subject: Feiertage ${DAVICALYEAR} eingetragen"
    echo "Feiertage für ${DAVICALYEAR} in davical eingetragen:"
    echo ""
    cat "${LOG}"
) 2>&1 | /usr/lib/sendmail -t

###
# habe fertig
exit

